﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections;
using System.ComponentModel.DataAnnotations;
using System.Collections.Generic;
namespace XamGridValidation
{
    public class TestDataBase : INotifyPropertyChanged
    {
        #region Members

        private string _stringData;
        private bool _boolData;
        private DateTime _dateTimeData;

        protected int _integerData = 0;
        static private int _count = 0;
        private Dictionary<string, string> _propertyErrors;

        #endregion

        #region Constructors
        public TestDataBase()
        {
            InitDataErrorInfo();
            InitProperties();
        }
        #endregion

        #region Methods
        private void InitProperties()
        {
            this.StringData = string.Format("StringData{0}", TestDataBase._count++);
        }

        public void ClearDataErrorInfo(string propertyName)
        {
            AssertThisHasPropertyWithName(propertyName);
            _propertyErrors[propertyName] = string.Empty;
            OnErrorsChanged(new DataErrorsChangedEventArgs(propertyName));
        }

        public void SetError(string propertyName, string error)
        {
            AssertThisHasPropertyWithName(propertyName);
            _propertyErrors[propertyName] = error;
            OnErrorsChanged(new DataErrorsChangedEventArgs(propertyName));
        }

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        protected void AssertThisHasPropertyWithName(string propertyName)
        {
            if (!_propertyErrors.ContainsKey(propertyName))
            {
                throw new ArgumentException(string.Format("No property named {0} on {1}."
                    , propertyName, this.GetType().FullName));
            }
        }

        protected void OnErrorsChanged(DataErrorsChangedEventArgs args)
        {
            if (ErrorsChanged != null)
                ErrorsChanged(this, args);
        }

        private void InitDataErrorInfo()
        {
            var properties = this.GetType().GetProperties();
            _propertyErrors = new Dictionary<string, string>();

            foreach (var propertyInfo in properties)
            {
                _propertyErrors.Add(propertyInfo.Name, string.Empty);
            }
        }

        #endregion

        #region Properties

        public string StringData
        {
            get
            {
                return _stringData;
            }
            set
            {
                _stringData = value;
                OnPropertyChanged("StringData");
            }
        }

        public int IntegerData
        {
            get
            {
                return _integerData;
            }
            set
            {
                _integerData = value;
                OnPropertyChanged("IntegerData");
            }
        }

        public bool BoolData
        {
            get
            {
                return _boolData;
            }
            set
            {
                _boolData = value;
                OnPropertyChanged("BoolData");
            }
        }

        public DateTime DateTimeData
        {
            get { return _dateTimeData; }
            set { _dateTimeData = value; OnPropertyChanged("DateTimeData"); }
        }

        #endregion //Properties

        #region Data Error Info

        [Display(AutoGenerateField = false)]
        public string this[string propertyName]
        {
            get
            {
                AssertThisHasPropertyWithName(propertyName);
                string result = null;
                if (_propertyErrors[propertyName] == string.Empty)
                    return result;

                return _propertyErrors[propertyName];
            }
        }

        [Display(AutoGenerateField = false)]
        public string Error
        {
            get
            {
                string errors = string.Empty;
                foreach (var propertyError in _propertyErrors)
                {
                    if (propertyError.Value == string.Empty) continue;
                    errors += string.Format("{0}: {1}\n", propertyError.Key, this[propertyError.Key]);
                }
                return errors.ToString().Trim();
            }
        }

        #endregion //Data Error Info

        #region INotifyDataErrorInfo Members

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        public IEnumerable GetErrors(string propertyName)
        {
            if (string.IsNullOrEmpty(propertyName))
                yield break;
            if (_propertyErrors[propertyName] != string.Empty)
                yield return _propertyErrors[propertyName];
            else
                yield break;
        }

        [Display(AutoGenerateField = false)]
        public bool HasErrors
        {
            get
            {
                foreach (var el in _propertyErrors)
                    if (el.Value != string.Empty)
                        return true;
                return false;
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion
    }

    public class TestIDataErrorInfo : TestDataBase, IDataErrorInfo
    { }
    public class TestINotifyDataErrorInfo : TestDataBase, INotifyDataErrorInfo
    { }
    public class TestBothInterfaces : TestDataBase, INotifyDataErrorInfo, IDataErrorInfo
    { }
}